import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.lang.Thread;
import java.util.Arrays;

public class ThreadOuter extends Thread
{
    public ThreadOuter(ThreadOuter spawner, int dim, int r_start, int r_end, int c_start, int c_end)
    {
        this.r_start = r_start;
        this.r_end = r_end;
        this.c_start = c_start;
        this.c_end = c_end;
        this.spawner = spawner;
        this.dim = dim;
        outerProduct = new int[dim][dim];
        done = false;
    }
    public void run()
    {
        // subproblems are 4 block matrices
        ThreadOuter topLeft = null;
        ThreadOuter topRight = null;
        ThreadOuter bottomLeft = null;
        ThreadOuter bottomRight = null;
        done = false;
        if(r_start == r_end && c_start == c_end)
        {
            outerProduct[0][0] = v[r_start]*w[c_start];  // just one element!
        }
        else
        {
            // subproblems are 4 block matrices
            int r_mid = (r_start + r_end) / 2;
            int c_mid = (c_start + c_end) / 2;
            dim = r_mid - r_start + 1;

            topLeft = new ThreadOuter(this, dim, r_start, r_mid, c_start, c_mid);
            topRight = new ThreadOuter(this, dim, r_start, r_mid, c_mid + 1, c_end);
            bottomLeft = new ThreadOuter(this, dim, r_mid + 1, r_end, c_start, c_mid);
            bottomRight = new ThreadOuter(this, dim, r_mid + 1, r_end, c_mid + 1, c_end);

            topLeft.start();
            topRight.start();
            bottomLeft.start();
            bottomRight.start();
        }
        if(r_start != r_end || c_start != c_end){
            topLeft.sync();
            topRight.sync();
            bottomLeft.sync();
            bottomRight.sync();

            outerProduct = mergeBlocks( topLeft.outerProduct,
                                        topRight.outerProduct,
                                        bottomLeft.outerProduct,
                                        bottomRight.outerProduct);
        }
        done = true;
        if(spawner == null)
        {
          //Display the outerProduct, properly formatted.
          for (int i = 0; i < outerProduct.length; i++) {
            System.out.print(outerProduct[i][0]);
            for (int j = 1; j < outerProduct.length; j++) {
              System.out.print(","+outerProduct[i][j]);
            }
            System.out.println();
          }
        }
    }
    public synchronized void sync()
    {
        if(!done)
        {
            try
            {
                wait();
            }
            catch(InterruptedException ie)
            {
                ie.printStackTrace();
            }
        }
    }
    public static void main(String args[])
    {
        String vs = "";
        String ws = "";
        // Read in data
        try {
          BufferedReader in = new BufferedReader(new FileReader(args[0]));
          String s = in.readLine();
          String[] nums;

          do {
            nums = s.split(",");
            vs += nums[0]+",";
            ws += nums[1]+",";
          } while ((s = in.readLine()) != null);

          in.close();
        } catch (IOException e) {
          System.out.println("some error");
        }

        v = Arrays.stream(vs.split(",")).mapToInt(Integer::parseInt).toArray();
        w = Arrays.stream(ws.split(",")).mapToInt(Integer::parseInt).toArray();

        int d = v.length;
        ThreadOuter parent = new ThreadOuter(null, d, 0, d - 1, 0, d - 1);
        startTime = System.nanoTime();
        parent.start();
        endTime = System.nanoTime();

        // put any argument after data to display execution time
        if (args.length > 1) {
          try
          {
              Thread.sleep(1800);
          }
          catch(InterruptedException ex)
          {
              Thread.currentThread().interrupt();
          }
          System.out.println();
          System.out.print("Total Execution Time: " + (endTime - startTime));
          System.out.println(" [ns]");
        }
    }


    public static int[][] mergeBlocks(int[][] a, int[][] b, int[][] c, int[][] d) {
        // block length
        int l = a.length;
        int n = 2*l;
        int[][] ret = new  int[n][n];

        for(int i = 0; i < l; i++) {
          for (int j = 0; j < l; j++) {
            ret[i][j] = a[i][j];
            ret[i][j+l] = b[i][j];
            ret[i+l][j] = c[i][j];
            ret[i+l][j+l] = d[i][j];
          }
        }
        return ret;
    }


    // row and column start and end indices
    int r_start;
    int r_end;
    int c_start;
    int c_end;
    int dim;
    // int c = 0;
    // Need to return a matrix, not an inner product
    int[][] outerProduct;
    ThreadOuter spawner;
    boolean done;
    static int[] v;
    static int[] w;
    static long startTime;
    static long endTime;

}
